/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package org.wso2.registry.config;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.wso2.registry.RegistryException;
import org.wso2.registry.i18n.Messages;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;

public class RegistryConfiguration {

    private Map dbconfgigMap = new HashMap();

    /**
     * This method will read the regisrty.xml and using the data given in the file
     * resgistry configuratoon will be populated. And if the xmlLocation is null then it
     * will use the one in the class path.
     *
     * @param xmlLocation : Location of the
     * @throws org.wso2.registry.RegistryException : if something went wrong
     */
    public void populateRegistryConfig(String xmlLocation) throws RegistryException {
        InputStream in;
        if (xmlLocation != null) {
            try {
                in = new FileInputStream(xmlLocation);
            } catch (FileNotFoundException e) {
                throw new RegistryException(e.getMessage());
            }
        } else {
            in = Thread.currentThread().getContextClassLoader().getResourceAsStream("registry.xml");
            if (in == null) {
                throw new RegistryException(Messages.getMessage("inputstream.null", "registry.xml"));
            }
        }
        try {
            StAXOMBuilder buillder = new StAXOMBuilder(in);
            OMElement configElement = buillder.getDocumentElement();
            if (configElement != null) {
                Iterator dbconfgs = configElement.getChildrenWithName(new QName("dbconfig"));
                while (dbconfgs.hasNext()) {
                    OMElement dbconfig = (OMElement) dbconfgs.next();
                    DataBaseConfiguration dataBaseConfiguration = new DataBaseConfiguration();
                    String dbName = dbconfig.getAttributeValue(new QName("name"));
                    if(dbName == null){
                        throw new RegistryException(Messages.getMessage("dbname.null"));
                    }
                    dataBaseConfiguration.setConfigName(dbName);
                    OMElement userName = dbconfig.getFirstChildWithName(new QName("userName"));
                    if (userName != null) {
                        dataBaseConfiguration.setUserName(userName.getText());
                    }
                    OMElement password = dbconfig.getFirstChildWithName(new QName("password"));
                    if (password != null) {
                        dataBaseConfiguration.setPassWord(password.getText());
                    }
                    OMElement url = dbconfig.getFirstChildWithName(new QName("url"));
                    if (url != null) {
                        dataBaseConfiguration.setDbUrl(url.getText());
                    }
                    OMElement driverName = dbconfig.getFirstChildWithName(new QName("driverName"));
                    if (driverName != null) {
                        dataBaseConfiguration.setDriverName(driverName.getText());
                    }
                    dbconfgigMap.put(dbName,dataBaseConfiguration);
                }
            }

        } catch (XMLStreamException e) {
            throw new RegistryException(e.getMessage());
        }
    }

    public static void main(String[] args) throws Exception {
        RegistryConfiguration configuration = new RegistryConfiguration();
        configuration.populateRegistryConfig(null);
    }
    public DataBaseConfiguration getDataBaseConfiguration(String configName){
        return (DataBaseConfiguration) dbconfgigMap.get(configName);
    }
}
